/*
 * Copyright (c) 1990,1993 Regents of The University of Michigan.
 * All Rights Reserved.  See COPYRIGHT.
 */

#ifndef AFPD_AFP_UTIL_H
#define AFPD_AFP_UTIL_H 1

/* afp_util.c */
extern const char *AfpNum2name(int);
extern const char *AfpErr2name(int err);

#endif
